'use strict';

var Boleto = require('./boleto');

module.exports = function () {

    var ItauCripto = {};

    ItauCripto.__sbox = [];
    ItauCripto.__key = [];

    ItauCripto.__inicializa = function (paramString) {
        var m = (paramString || '').length;

        for (var j = 0; j <= 255; j++) {
            ItauCripto.__key[j] = paramString.substr((j % m), 1);
            ItauCripto.__sbox[j] = j;
        }

        var k = 0;
        for (j = 0; j <= 255; j++) {
            k = (k + ItauCripto.__sbox[j] + ItauCripto.__key[j].charCodeAt(0)) % 256;
            var i = ItauCripto.__sbox[j];
            ItauCripto.__sbox[j] = ItauCripto.__sbox[k];
            ItauCripto.__sbox[k] = i;
        }
    };

    ItauCripto.__algoritmo = function (dados, chave) {
        chave = (chave || '').trim().toUpperCase();
        dados = dados || '';

        var k = 0;
        var m = 0;
        var str = '';

        ItauCripto.__inicializa(chave);

        for (var j = 1; j <= dados.length; j++) {
            k = (k + 1) % 256;
            m = (m + ItauCripto.__sbox[k]) % 256;
            var i = ItauCripto.__sbox[k];

            ItauCripto.__sbox[k] = ItauCripto.__sbox[m];
            ItauCripto.__sbox[m] = i;

            var n = ItauCripto.__sbox[((ItauCripto.__sbox[k] + ItauCripto.__sbox[m]) % 256)];
            var i1 = (dados.substr((j - 1), 1).charCodeAt(0) ^ n);
            str += String.fromCharCode(i1);
        }

        return str;
    };

    ItauCripto.__getRandom = function (min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    };

    ItauCripto.__converte = function (paramString) {
        var random = ItauCripto.__getRandom(0, 999999999) / 1000000000;
        var c = String.fromCharCode(26.0 * random + 65.00);

        var str = c.toString();
        for (var i = 0; i < paramString.length; i++) {
            var k = paramString.substr(i, 1);
            var j = k.charCodeAt(0);
            str += j;

            random = ItauCripto.__getRandom(0, 999999999) / 1000000000;
            c = String.fromCharCode(26.0 * random + 65.00);
            str += c;
        }
        return str;
    };

    ItauCripto.addBoleto = function (cpfCnpj, nomeSacado, cep, pedido, dataVencimento, valor) {
        var boleto = new Boleto();

        if (cpfCnpj) {
            boleto.setCpfCnpj(cpfCnpj);
        }

        if (nomeSacado) {
            boleto.setNomeSacado(nomeSacado);
        }

        if (cep) {
            boleto.setCepSacado(cep);
        }

        if (pedido) {
            boleto.setPedido(pedido);
        }

        if (dataVencimento) {
            boleto.setDataVencimento(dataVencimento);
        }

        if (valor) {
            boleto.setValor(valor);
        }

        return boleto;
    };

    /**
     * Gera criptografia para emissão do boleto
     * @param codEmp Código da empresa
     * @param chave Chave da empresa
     * @param boleto Boleto
     */
    ItauCripto.geraCripto = function (codEmp, chave, boleto) {
        if ((codEmp || '').length != 26) {
            throw new Error('Tamanho do codigo da empresa diferente de 26 posições.');
        }

        if ((chave || '').length != 16) {
            throw new Error('Tamanho do codigo da empresa diferente de 16 posições.');
        }

        if (!boleto || !boleto.isValid()) {
            throw new Error('Boleto inválido');
        }

        ItauCripto.__sbox = [];
        ItauCripto.__key = [];

        var str1 = ItauCripto.__algoritmo(boleto.getLinhaBoletoAlgoritmo(), chave);
        var str2 = ItauCripto.__algoritmo(codEmp.trim().toUpperCase() + str1, 'SEGUNDA12345ITAU');
        var converte = ItauCripto.__converte(str2);

        return converte;
    };

    /**
     * Gera criptografia para consulta de Boleto
     * @param codEmp Código da empresa
     * @param pedido Código do pedido
     * @param formato 0 - HTML, 1 - XML
     * @param chave Chave da empresa
     */
    ItauCripto.geraConsulta = function (codEmp, pedido, formato, chave) {
        if ((codEmp || '').length != 26) {
            throw new Error('Tamanho do codigo da empresa diferente de 26 posições.');
        }

        if ((chave || '').length != 16) {
            throw new Error('Tamanho do codigo da empresa diferente de 16 posições.');
        }

        var boleto = new Boleto();
        boleto.setPedido(pedido);

        ItauCripto.__sbox = [];
        ItauCripto.__key = [];

        var str1 = ItauCripto.__algoritmo(boleto.getPedido() + formato, chave);
        var str2 = ItauCripto.__algoritmo(codEmp.trim().toUpperCase() + str1, 'SEGUNDA12345ITAU'); //?
        var converte = ItauCripto.__converte(str2);

        return converte;
    };

    return ItauCripto;
}();
